#! /bin/bash

# If information base is a text file, name it here, else let it empty
INFILE=""

# Where the protocol of not proper configured domains should go
OUTFILE="obsolete_domains.txt"

# ip addresses of mail host which the domains are to be checked against (multiple separated by space).
# Important: To avoid false positives, name ALL ip addresses where the mail host is reachable!!
MYADDRS=""

# If information base is a directory, put the dir to scan for domains for, else leave empty
MAILROOT=""

# This is for define a domain that should be excluded from checking (multiple seperated by space)
# e.g. a directory that doesn't contain a domain, like lost+found or a new not yet registered one
EXCLUDES="uidcatchall"

# If Bernstein's (djb)'s DNS Tools are installed instead of them from ISC
#DIGTOOL="djb"

# Set this to 1 if you like some verbose output when running
VERBOSE=1

function check_domain { 
  domain=$1
  [ "$VERBOSE" == "1" ] && echo -n "Checking Domain: $domain "
  if [ "$DIGTOOL" == "djb" ]
  then
    mx=`dnsmx $domain`
  else
    mx=`dig +short mx $domain`
  fi
  for specialdir in $EXCLUDES
  do
    if [ "$domain" == "$specialdir" ]
    then
      [ "$VERBOSE" == "1" ] && echo -n "- excluded, not checking"
      return
    fi
  done
  if echo $mx | egrep -qv "[a-z0-9_]+"
  then
    [ "$VERBOSE" == "1" ] && echo -n "- no MX record, logging and skipping..."
    echo "$domain:none:none" >> $OUTFILE
    return
  fi
  localaddr=0
  addrs=""
  mx=$(echo $mx)
  for mailhost in $mx 
  do
    if [ "$DIGTOOL" == "djb" ]
    then
      arecs=`dnsqr A $mailhost | grep "answer:" | cut -d " " -f 5`
    else
      arecs=`dig +short A $mailhost`
    fi
    for arec in $arecs
    do
     for ip in $MYADDRS
      do
        if [ "$arec" == "$ip" ]
        then
          localaddr=1
        fi
      done
      [ "$addrs" == "" ] && addrs="$arec" || addrs="$addrs $arec"
    done
  done
  if [ "$localaddr" == "0" ]
  then
    [ "$VERBOSE" == "1" ] && echo -n "- is foreign, logging..."
    echo "$domain:$mx:$addrs" >> $OUTFILE
  else
    [ "$VERBOSE" == "1" ] && echo -n "- is local and OK"
  fi
}

# Main routine
echo "Domain:MX Records:A Records" > $OUTFILE
if [ "$INFILE" != "" ]
then
  cat $INFILE | while read x
  do
    check_domain $x
    [ "$VERBOSE" == "1" ] && echo
  done
else
  find $MAILROOT/* -maxdepth 0 -exec basename {} \; | while read x
  do
    check_domain $x 
    [ "$VERBOSE" == "1" ] && echo
  done
fi

